/**************************************************************************************************************************
// This script was authored by Lorne Brooks - Email: lbtools2k@gmail.com
//
//
// REVISION HISTORY:
// Version 1.3 27/10/14;
//						- Email change for help contact.
// Version 1.2 23/09/14;
//						- Improve code for finding rotation keys on a selected CAT node. Now handles all CAT node types.
// Version 1.1 22/09/14;
//						- Fixed the snapshot functionality. Do longer selects anything which means it works for frozen objects in Max 2015.
//						Also, now handles multiple skin meshes in the Mesh Character layer. Only Skin meshes are snapshot.
// Version 1.0 19/09/14;
//						- First public release.
//
//
// This script is not copyrighted, please feel free to modify or distribute.
//
**************************************************************************************************************************/

global g_LBTools_CPG

struct s_LBTools_catPoseGrabber
(

/*==================================================================================
// Structure variables
====================================================================================*/

CATnode,
catPoseGrabberPanelINI = getDir #plugcfg + "\catPoseGrabberPanel.ini",
a_cats = #(),
f_openCatPoseGrabberUI,
storePose_rollout,
aboutScript_rollout,
dPos,

/*==================================================================================
// Structure functions
/*==================================================================================

f_findSkinModifier
f_missingCatError
f_collectCatParents
f_validateCATnode
f_validateAnimReady
f_validateLayers
f_catPoseGrabber

====================================================================================*/

fn f_findSkinModifier obj =
(
	local skinFound = false
	for i = 1 to obj.modifiers.count do
	(
		if classof obj.modifiers[i] == Skin do
		(
			skinFound = true
			exit
		)
	)
	skinFound
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_missingCatError =
(
	messageBox "Error: A previously picked CAT Rig is missing from the scene - UI updated." title:""
	setIniSetting g_LBTools_CPG.catPoseGrabberPanelINI "CatPoseGrabberPanel" "CurrentRig" "1"
	g_LBTools_CPG.f_openCatPoseGrabberUI()
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_collectCatParents =
(
	for h in helpers where classOf h == CATParent do
	(
		append g_LBTools_CPG.a_cats h
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_validateCATnode =
(
	if NOT (isValidNode g_LBTools_CPG.CATnode) do
	(
		g_LBTools_CPG.CATnode = undefined
		messageBox "No CAT Parent was found in the scene.\n" title:""
		g_LBTools_CPG.f_openCatPoseGrabberUI()
		return false
	)
	return true
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_validateAnimReady quiet:false =
(
	setCommandPanelTaskMode #motion
	if g_LBTools_CPG.CATnode.numLayers > 0 then
	(
		if g_LBTools_CPG.CATnode.CATMode == 1 then
		(
			return true
		)
		else
		(
			if quiet == false do messagebox "Error: You must be in Animate mode (Green)." title:""
		)
	)
	else
	(
		if quiet == false do messagebox "Error: No layers were found." title:""
	)
	return false
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_validateLayers quiet:false =
(
	setCommandPanelTaskMode #motion
	if g_LBTools_CPG.CATnode.numLayers > 0 then
	(
		if g_LBTools_CPG.CATnode.CATMode == 1 then
		(
			if g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.layers.controller.selectedlayer] != undefined then
			(
				if g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.layers.controller.selectedlayer].LayerEnabled then
				(
					return true
				)
				else
				(
					if quiet == false do messagebox "Error: The currently selected layer must be Enabled." title:""
				)
			)
			else
			(
				if quiet == false do messagebox "Error: Select a layer first." title:""
			)
		)
		else
		(
			if quiet == false do messagebox "Error: You must be in Animate mode (Green)." title:""
		)
	)
	else
	(
		if quiet == false do messagebox "Error: No layers were found." title:""
	)
	return false
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_catPoseGrabber =
(
	rollout storePose_rollout "CAT Pose Grabber"
	(
		local storePosesFolder = (substituteString (getdir #plugcfg) @"\en-US\plugcfg" @"\plugcfg_ln\CAT\Poses\StorePoses\")
		local storePoseFile = (substituteString (getdir #plugcfg) @"\en-US\plugcfg" @"\plugcfg_ln\CAT\Poses\" + "StorePose.pse")
		local str_editBox = ""
		
		dropDownList ddl_cats "" width:155 across:2 align:#left tooltip:"Choose a CAT rig from the list for the panel to operate on"
		button btn_refreshUI "R" width:22 height:22 align:#right offset:[8,0] tooltip:"Refresh the list by searching for all CAT Parents in the scene"
		
		group "Single Poses"
		(
			label lbl_1 "Layer type to store a pose from:" align:#left
			radiobuttons rdbtns_layerType "" labels:#("Previous","Selected") default:2 offsets:#([-10,0,0],[10,0,0])
			button btn_storePose ">>  STORE BY LAYER TYPE  <<" width:180 height:18 align:#center tooltip:"Stores a copy of the whole rig as a pose from either the previous layer or selected layer"
			button btn_pasteSliderPoseRelative "PASTE SINGLE POSE - RELATIVE" width:180 height:18 tooltip:"Retrieves the stored pose and pastes onto the selected layer at the current slider time - with Relative transforms"
			button btn_pasteSliderPoseABS "PASTE SINGLE POSE - ABSOLUTE" width:180 height:18 tooltip:"Retrieves the stored pose and pastes onto the selected layer at the current slider time - with Absolute transforms"
			checkbox chbx_keyed "Keyed" checked:true align:#left across:2 tooltip:"Toggle OFF to paste the pose without setting keys"
			checkbox chbx_mirrored "Mirrored" checked:false align:#right tooltip:"Pose will be loaded using CAT's own mirroring method"
		)
		group "Multiple Poses"
		(
			label lbl_frames "Frames To Store:" align:#left offset:[5,0]
			editText edtxt_frames "" text:"" across:2 width:155
			button btn_clear "x" width:16 height:16 offset:[0,0] align:#right toolTip:"Clear the editbox"
			button btn_fullRange "FULL RANGE" height:20 align:#left offset:[5,0] across:3 toolTip:"Add the frames for the full time range into the editbox"
			button btn_RotKeys "ROT KEYS" height:20 offset:[30,0] toolTip:"Adds all the frames from the selected object rotation keys into the editbox"
			checkbutton chkbtn_help "?" width:20 height:20 align:#right toolTip:"Display a help dialog explaining the multiple poses editbox"
			groupBox gpbx_1 "" pos:[10,194] width:180 height:80
			button btn_storePoses ">>  STORE DEFINED POSES  <<" width:176 height:18 tooltip:"Poses from the current layer as defined in the edit box are stored"
			button btn_pastePosesSelectedLayer "PASTE POSES - SELECTED" width:175 height:18 tooltip:"Poses as were defined in the edit box are pasted onto the selected layer"
			button btn_pastePosesABS "PASTE POSES - NEW ABS" width:175 height:18 tooltip:"Poses as were defined in the edit box are pasted onto a new ABS layer"
			button btn_pastePosesRelative "PASTE POSES - NEW LOCAL" width:175 height:18 align:#left tooltip:"Poses as were defined in the edit box are pasted onto a new Local Adjustment layer"
			button btn_pastePosesWorld "PASTE POSES - NEW WORLD" width:175 height:18 align:#left tooltip:"Poses as were defined in the edit box are pasted onto a new World Adjustment layer"
		)
		group "Snapshot ('Mesh Character' layer)"
		(
			button btn_snapshot ">>  SNAP CURRENT FRAME  <<" width:175 height:18 align:#center tooltip:"Creates an combined snapshot of all skinned meshes at the current slider time and makes it unobtrusive in the viewport"
			button btn_hideSnapshots "HIDE / UNHIDE" height:18 across:2 align:#left tooltip:"Hides or unhides the SNAPSHOTS scene layer"
			button btn_deleteSnapshots "DELETE" height:18 align:#right tooltip:"Deletes the SNAPSHOTS scene layer and its contents"
		)
		button btn_about "About" align:#center
		
		rollout info_rollout "CAT Pose Grabber Help"
		(
			group ""
			(
				label lbl_heading "----  Multiple Poses  ----" align:#center
				label lbl_description1 "Allows the creation of a new layer"
				label lbl_description2 "using chosen poses entered into the"
				label lbl_description3 "edit box. These are copied from the"
				label lbl_description4 "currently selected layer."
				label lbl_gap1 "==================="
				label lbl_editbox "----  Edit Box  ----" align:#center
				label lbl_hint1 "Enter single frames or ranges"
				label lbl_hint2 "with a space between each"
				label lbl_hint3 "and a dash between the range."
				label lbl_hintEg "E.g. 0 5-8 50-75 250 1000"
			)
			
			on info_rollout close do
			(
				g_LBTools_CPG.storePose_rollout.chkbtn_help.checked = false
			)
		) --// end rollout
			
		on btn_about pressed do
		(
			try(destroyDialog g_LBTools_CPG.aboutScript_rollout)catch()
			createDialog g_LBTools_CPG.aboutScript_rollout
		)
		
		on storePose_rollout close do
		(
			try(destroyDialog info_rollout)catch()
			try(destroyDialog g_LBTools_CPG.aboutScript_rollout)catch()
		)
		
		on chkbtn_help changed state do
		(
			if state then
			(
				try(destroyDialog info_rollout)catch()
				createDialog info_rollout pos:(g_LBTools_CPG.dPos + [220, 100]) width:200
			)
			else
			(
				try(destroyDialog info_rollout)catch()
			)
		)	
		
		on storePose_rollout open do
		(
			if NOT (doesFileExist g_LBTools_CPG.catPoseGrabberPanelINI) then
			(
				setIniSetting g_LBTools_CPG.catPoseGrabberPanelINI "CatPoseGrabberPanel" "CurrentRig" "1"
				g_LBTools_CPG.f_openCatPoseGrabberUI()
			)
			else
			(
				g_LBTools_CPG.a_cats = #()
				g_LBTools_CPG.f_collectCatParents()
				if g_LBTools_CPG.a_cats.count > 0 then
				(
					local a_catNames = #()
					for b in g_LBTools_CPG.a_cats do
					(
						append a_catNames b.name
					)
					ddl_cats.items = a_catNames
					ddl_cats.selection = execute (getIniSetting g_LBTools_CPG.catPoseGrabberPanelINI "CatPoseGrabberPanel" "CurrentRig")
					if ddl_cats.selection > 0 then
					(
						g_LBTools_CPG.CATnode = g_LBTools_CPG.a_cats[ddl_cats.selection]
						ddl_cats.height = (ddl_cats.items.count + 1) * 100
						if g_LBTools_CPG.CATnode.RootHub != undefined then
						(
							select g_LBTools_CPG.CATnode.RootHub.node
						)
						else
						(
							messageBox "Error: No hub found associated with the CAT Parent. Create a Pelvis node." title:""
							setCommandPanelTaskMode #modify
							try(destroyDialog g_LBTools_CPG.storePose_rollout)catch()
						)
					)
					else
					(
						g_LBTools_CPG.f_missingCatError()					
					)
				)
				else
				(
					messageBox "Error: No CAT Parent found in the scene." title:""
					setIniSetting g_LBTools_CPG.catPoseGrabberPanelINI "CatPoseGrabberPanel" "CurrentRig" "1"
					try(destroyDialog g_LBTools_CPG.storePose_rollout)catch()
				)
			)
		)
		
		on ddl_cats selected val do
		(
			if isValidNode g_LBTools_CPG.a_cats[ddl_cats.selection] then
			(
				if ddl_cats.selected == g_LBTools_CPG.a_cats[ddl_cats.selection].name then
				(
					g_LBTools_CPG.CATnode = g_LBTools_CPG.a_cats[ddl_cats.selection]
					setIniSetting g_LBTools_CPG.catPoseGrabberPanelINI "CatPoseGrabberPanel" "CurrentRig" (ddl_cats.selection as string)
					try(select g_LBTools_CPG.CATnode.RootHub.node)catch(messageBox "Error: Missing hub root (Pelvis)." title:"")
				)
				else
				(
					g_LBTools_CPG.f_missingCatError()
				)
			)
			else
			(
				g_LBTools_CPG.f_missingCatError()
			)
		)
		
		on btn_refreshUI pressed do
		(
			g_LBTools_CPG.f_openCatPoseGrabberUI()
		)
		
		on storePose_rollout moved val do
		(
			g_LBTools_CPG.dPos = val
		)
		
		on btn_storePose pressed do
		(
			if g_LBTools_CPG.f_validateCATnode() do
			(
				if g_LBTools_CPG.f_validateLayers() do
				(
					if rdbtns_layerType.state == 1 do --// Previous layer type
					(
						if g_LBTools_CPG.CATnode.numlayers > 1 then
						(
							if g_LBTools_CPG.CATnode.layers.controller.selectedlayer == g_LBTools_CPG.CATnode.numLayers then --// Operate on the last (bottom most) layer
							(
								g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].layerEnabled = false --// Deactivate the working layer
								g_LBTools_CPG.CATnode.layers.controller.selectedlayer -= 1 --// Switch to the previous layer
								g_LBTools_CPG.CATnode.SavePose storePoseFile --// Copy the pose
								format "CATrig: % - Pose stored from layer > % < at frame: %\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name sliderTime
								g_LBTools_CPG.CATnode.layers.controller.selectedlayer += 1--// Switch back to the working layer
								g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].layerEnabled = true --// Activate the working layer
							)
							else
							(
								messageBox "Error: For PREVIOUS layer storage...\n\nMake sure you have the last (bottom most) layer selected.\n\nThis will be your 'working' layer for pasting the pose onto from the previous layer." title:""
							)
						)
						else
						(
							messageBox "Error: For PREVIOUS layer storage...\n\nMake sure there are at least TWO layers in the stack.\n\nNote: Layer names are not important." title:""
						)
					)
					if rdbtns_layerType.state == 2 do --// Selected layer type
					(
						g_LBTools_CPG.CATnode.SavePose storePoseFile --// Copy the pose
						format "CATrig: % - Pose stored from layer > % < at frame: %\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name sliderTime
					)
				)
			)
		)

		on btn_pasteSliderPoseRelative pressed do
		(
			if g_LBTools_CPG.f_validateCATnode() do
			(
				if g_LBTools_CPG.f_validateLayers() do
				(
					if chbx_keyed.checked then
					(
						with Animate on
						(
							if chbx_mirrored.checked then
							(
								g_LBTools_CPG.CATnode.LoadPose storePoseFile sliderTime transformdata:true mirrordata:true
								format "Applied to CATrig: % - Stored pose pasted (mirrored) on layer - % - at frame : %\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name sliderTime
							)
							else
							(
								g_LBTools_CPG.CATnode.LoadPose storePoseFile sliderTime transformdata:true
								format "Applied to CATrig: % - Stored pose pasted on layer > % < at frame : %\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name sliderTime
							)
						)
					)
					else
					(
						if chbx_mirrored.checked then
						(
							g_LBTools_CPG.CATnode.LoadPose storePoseFile sliderTime transformdata:true mirrordata:true
							format "Applied to CATrig: % - Stored pose pasted (mirrored) on layer - % - at frame : %\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name sliderTime
						)
						else
						(
							g_LBTools_CPG.CATnode.LoadPose storePoseFile sliderTime transformdata:true
							format "Applied to CATrig: % - Stored pose pasted on layer > % < at frame : %\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name sliderTime
						)
					)
					try(select g_LBTools_CPG.CATnode.RootHub.node)catch(messageBox "Error: Missing hub root (Pelvis)." title:"")
				)
			)
		)
		
		on btn_pasteSliderPoseABS pressed do
		(
			if g_LBTools_CPG.f_validateCATnode() do
			(
				if g_LBTools_CPG.f_validateLayers() do
				(
					if chbx_keyed.checked then
					(
						with Animate on
						(
							if chbx_mirrored.checked then
							(
								g_LBTools_CPG.CATnode.LoadPose storePoseFile sliderTime transformdata:false mirrordata:true
								format "Applied to CATrig: % - Stored pose pasted (mirrored) on layer - % - at frame : %\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name sliderTime
							)
							else
							(
								g_LBTools_CPG.CATnode.LoadPose storePoseFile sliderTime transformdata:false
								format "Applied to CATrig: % - Stored pose pasted on layer > % < at frame : %\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name sliderTime
							)
						)
					)
					else
					(
						if chbx_mirrored.checked then
						(
							g_LBTools_CPG.CATnode.LoadPose storePoseFile sliderTime transformdata:false mirrordata:true
							format "Applied to CATrig: % - Stored pose pasted (mirrored) on layer - % - at frame : %\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name sliderTime
						)
						else
						(
							g_LBTools_CPG.CATnode.LoadPose storePoseFile sliderTime transformdata:false
							format "Applied to CATrig: % - Stored pose pasted on layer > % < at frame : %\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name sliderTime
						)
					)
					try(select g_LBTools_CPG.CATnode.RootHub.node)catch(messageBox "Error: Missing hub root (Pelvis)." title:"")
				)
			)
		)
		
		on btn_clear pressed do
		(
			str_editBox = ""
			edtxt_frames.text = ""
			setFocus g_LBTools_CPG.storePose_rollout.edtxt_frames
		)
		
		on edtxt_frames changed str do
		(
			str_editBox= str
		)
		
		on btn_fullRange pressed do
		(
			local startTime = ((animationRange.start) as integer / ticksperframe) as string
			local endTime = ((animationRange.end) as integer / ticksperframe) as string
			str_editBox = startTime + "-" + endTime
			edtxt_frames.text = str_editBox
		)
		
		on btn_RotKeys pressed do
		(
			if g_LBTools_CPG.f_validateLayers() AND selection.count > 0 then
			(
				local obj = selection[1]
				str_editBox = ""
				edtxt_frames.text = ""
				
				local validFound = false
				if NOT validFound do
				(
					try --// Gizmo shapes
					(
						for i = 1 to obj.controller.targetController.LayerTrans.selectedLayerCtrl[2].controller.keys.count do
						(
							append str_editBox (((obj.controller.targetController.LayerTrans.selectedLayerCtrl[2].controller.keys[i].time  as integer / ticksperframe) as string) + " ")
						)
						validFound = true
					)catch()
				)
				
				if NOT validFound do
				(
					try --// Collarbones, Arb bones and Hubs
					(
						for i = 1 to obj.transform.controller[1].selectedLayerCtrl[2].controller.keys.count do
						(
							append str_editBox (((obj.transform.controller[1].selectedLayerCtrl[2].controller.keys[i].time  as integer / ticksperframe) as string) + " ")
						)
						validFound = true
					)catch()
				)
				
				if NOT validFound do
				(
					try --// Limb bones other than collarbones
					(
						for i = 1 to obj.transform.controller[2].selectedLayerCtrl[2].controller.keys.count do
						(
							append str_editBox (((obj.transform.controller[2].selectedLayerCtrl[2].controller.keys[i].time  as integer / ticksperframe) as string) + " ")
						)
						validFound = true
					)catch()
				)
				
				edtxt_frames.text = str_editBox
			)
			else
			(
				messageBox "Error: Select a CAT bone with rotation keys first." title:""
			)
		)
		
		on btn_storePoses pressed do
		(
			if g_LBTools_CPG.f_validateCATnode() do
			(
				if g_LBTools_CPG.f_validateLayers() do
				(
					try
					(
						if str_editBox != "" then
						(
							makeDir storePosesFolder --// In case of first time use
							--// Delete the folder contents
							files = getFiles (storePosesFolder + "*.*")
							for f in files do
							(
								deleteFile f
							)
							--// Store the poses based on the user edit box contents
							a_poseTimes = filterString str_editBox " "
							for item in a_poseTimes do
							(
								if findString item "-" != undefined then --// it's a range
								(
									local a_timeRange = filterString item "-"
									for t = (a_timeRange[1] as integer) to (a_timeRange[2] as integer) do
									(
										sliderTime = t
										g_LBTools_CPG.CATnode.SavePose (storePosesFolder + "pose_"  + (t as string) + ".pse")
									)
								)
								else --// it's a sinlge frame
								(
									sliderTime = (item as integer)
									g_LBTools_CPG.CATnode.SavePose (storePosesFolder + "pose_"  + item + ".pse")
								)
							)
							local str = "Multiple poses on CATrig: > " + g_LBTools_CPG.CATnode.catname + " < have been stored.\n"
							format str
							messageBox str title:""
						)
						else
						(
							messageBox "Error: First enter the pose frames you want to store into the edit box." title:""
						)
					)
					catch
					(
						messageBox "Error: Make sure the string entered in the edit box has spaces between each frame or a dash between a range start and end.\n\nE.g. 0 50-100 250 1000" title:""
					)
				)
			)
		)

		on btn_pastePosesSelectedLayer pressed do
		(
			if g_LBTools_CPG.f_validateCATnode() do
			(
				if g_LBTools_CPG.f_validateLayers() do
				(
					makeDir storePosesFolder --// In case of first time use
					sliderTime = 0
					local a_poseFiles = getFiles (storePosesFolder + "*.*")
					for poseFile in a_poseFiles while NOT keyboard.escPressed do
					(
						local a = filterstring poseFile "_."
						sliderTime = a[a.count - 1] as time
						with Animate on
						(
							g_LBTools_CPG.CATnode.LoadPose poseFile sliderTime transformdata:false --// This must be false for ABS layer loading to work
						)
					)
					sliderTime = 0
					format "Applied to CATrig: % - Stored poses pasted on selected layer > % <.\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name
				)
			)
		)
		
		on btn_pastePosesABS pressed do
		(
			if g_LBTools_CPG.f_validateCATnode() do
			(
				if g_LBTools_CPG.f_validateAnimReady() do
				(
					makeDir storePosesFolder --// In case of first time use
					sliderTime = 0
					g_LBTools_CPG.CATnode.AppendLayer "Stored Poses ABS" #absolute
					local a_poseFiles = getFiles (storePosesFolder + "*.*")
					for poseFile in a_poseFiles while NOT keyboard.escPressed do
					(
						local a = filterstring poseFile "_."
						sliderTime = a[a.count - 1] as time
						with Animate on
						(
							g_LBTools_CPG.CATnode.LoadPose poseFile sliderTime transformdata:false --// This must be false for ABS layer loading to work
						)
					)
					sliderTime = 0
					format "Applied to CATrig: % - Stored poses pasted on a new ABS layer > % <.\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name
				)
			)
		)
		
		on btn_pastePosesRelative pressed do
		(
			if g_LBTools_CPG.f_validateCATnode() do
			(
				if g_LBTools_CPG.f_validateAnimReady() do
				(
					makeDir storePosesFolder --// In case of first time use
					sliderTime = 0
					g_LBTools_CPG.CATnode.AppendLayer "Stored Poses L" #relativeLocal
					local a_poseFiles = getFiles (storePosesFolder + "*.*")
					for poseFile in a_poseFiles while NOT keyboard.escPressed do
					(
						local a = filterstring poseFile "_."
						sliderTime = a[a.count - 1] as time
						with Animate on
						(
							g_LBTools_CPG.CATnode.LoadPose poseFile sliderTime
						)
					)
					sliderTime = 0
					format "Applied to CATrig: % - Stored poses pasted on a new Local layer > % <.\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name
				)
			)
		)

		on btn_pastePosesWorld pressed do
		(
			if g_LBTools_CPG.f_validateCATnode() do
			(
				if g_LBTools_CPG.f_validateAnimReady() do
				(
					makeDir storePosesFolder --// In case of first time use
					sliderTime = 0
					g_LBTools_CPG.CATnode.AppendLayer "Stored Poses W" #relativeWorld
					local a_poseFiles = getFiles (storePosesFolder + "*.*")
					for poseFile in a_poseFiles while NOT keyboard.escPressed do
					(
						local a = filterstring poseFile "_."
						sliderTime = a[a.count - 1] as time
						with Animate on
						(
							g_LBTools_CPG.CATnode.LoadPose poseFile sliderTime
						)
					)
					sliderTime = 0
					format "Applied to CATrig: % - Stored poses pasted on a new World layer > % <.\n" g_LBTools_CPG.CATnode.catname g_LBTools_CPG.CATnode.layers.controller[g_LBTools_CPG.CATnode.selectedlayer].name
				)
			)
		)
		
		on btn_snapshot pressed do
		(
			local layerMeshChar = layerManager.getLayerFromName "Mesh Character"
			-- // First make sure the Mesh Character layer exists
			if layerMeshChar != undefined then
			(
				--// Collect all nodes from the scene layer
				local a_nodes = #()
				layerMeshChar.nodes &a_nodes
				if a_nodes.count > 0 then
				(
					--// Collect only the skin meshes from the scene layer nodes array
					local a_skinNodes = #()
					for n in a_nodes do
					(
						if (g_LBTools_CPG.f_findSkinModifier n) do
						(
							append a_skinNodes n
						)
					)
					if a_skinNodes.count > 0 then
					(
						--// Snapshot and attach all the skin mesh copies
						local snapMesh = snapShot a_skinNodes[1]
						for i = 2 to a_skinNodes.count do
						(
							attach snapMesh (snapshot a_skinNodes[i])
						)
						snapMesh.pivot = snapMesh.center
						-- // Create a new layer if this is the first snapshot
						if LayerManager.getLayerFromName "SNAPSHOTS" == undefined do
						(
							LayerManager.newLayerFromName "SNAPSHOTS"
						)
						-- // Add the new snapshot mesh to the snapshots layer, freeze it and set object properties transparency
						local layerSnapshots = LayerManager.getLayerFromName "SNAPSHOTS"
						layerSnapshots.addNode snapMesh
						freeze snapMesh
						snapMesh.backfacecull = on
						snapMesh.showFrozenInGray = off
						-- // Rename the snapshot mesh and add the slider time frame
						snapMesh.name = ("Snapshot :: " + sliderTime as string)
						-- // Assign a material which has a low opacity and grey colour
						snapMesh.material = meditMaterials[19]
						with Animate off
						(
							meditMaterials[19].Diffuse = color 150 150 150
							meditMaterials[19].ambient = color 150 150 150
							meditMaterials[19].opacity = 20
							meditMaterials[19].name = "See Through"
							snapMesh.wirecolor = color 150 150 150
						)
					)
					else
					(
						messageBox "Error: The scene layer ''Mesh Character'' should contain skinned meshes." title:""
					)
				)
				else
				(
					messageBox "Error: No nodes were found the scene layer ''Mesh Character''." title:""
				)
			)
			else
			(
				messageBox "Error: ''Mesh Character'' scene layer not found." title:""
			)
		)
		
		on btn_hideSnapshots pressed do
		(
			local layerSnapshots = layerManager.getLayerFromName "SNAPSHOTS"
			if layerSnapshots != undefined do
			(
				if layerSnapshots.isHidden then
				(
					layerSnapshots.isHidden = false
				)
				else
				(
					layerSnapshots.isHidden = true
				)
			)
		)		
		
		on btn_deleteSnapshots pressed do
		(
			local layerSnapshots = layerManager.getLayerFromName "SNAPSHOTS"
			if layerSnapshots != undefined do
			(
				layerSnapshots.nodes &nodesInLayer
				delete nodesInLayer
				layerManager.deleteLayerByName "SNAPSHOTS"
			)
		)
	) --// end rollout
	
	rollout aboutScript_rollout "About"
	(
		label lbl_author "Author - Lorne Brooks"
		hyperLink myEmail "lbtools2k@gmail.com" color:(color 0 0 255) hovercolor:(color 0 0 190) visitedcolor:(color 190 0 0) align:#center address:"mailto:lbtools2k@gmail.com"
		label lbl_version "Version 1.3 - 27 Oct 2014"
	) --// end rollout
	
	--// Create UI
	fn f_openCatPoseGrabberUI =
	(
		try(destroyDialog g_LBTools_CPG.storePose_rollout)catch()
		if g_LBTools_CPG.dPos == undefined do
		(
			g_LBTools_CPG.dPos = mouse.screenpos + [-75,-15]
		)
		createDialog g_LBTools_CPG.storePose_rollout width:200 pos:g_LBTools_CPG.dPos
		setFocus g_LBTools_CPG.storePose_rollout.edtxt_frames
	)
	
	g_LBTools_CPG.f_openCatPoseGrabberUI()
)

) --// end struct

g_LBTools_CPG = s_LBTools_catPoseGrabber()


/*********************************************************************************************************************************************************************/
--// MACRO CALLS FOR UI
/*********************************************************************************************************************************************************************/

macroscript catPoseGrabber
	category:"LB Tools"
	buttontext:"{ CPG }"
	tooltip:"CAT Pose Grabber 1.3"
(	
	g_LBTools_CPG.f_catPoseGrabber()
)

/*********************************************************************************************************************************************************************/